unit clMultiStream;

interface

uses
  Classes;

type
  TMultiStream = class(TStream)
  private
    FPosition: Integer;
    FList: TList;
    function GetStream(Index: Integer): TStream;
    function GetTotalSize: Longint;
  protected
    procedure SetSize(NewSize: Longint); override;
  public
    constructor Create;
    destructor Destroy; override;
    procedure AddStream(AStream: TStream);
    function Read(var Buffer; Count: Longint): Longint; override;
    function Seek(Offset: Longint; Origin: Word): Longint; override;
    function Write(const Buffer; Count: Longint): Longint; override;
  end;


implementation

{ TMultiStream }

procedure TMultiStream.AddStream(AStream: TStream);
begin
  FList.Add(AStream);
end;

constructor TMultiStream.Create;
begin
  inherited Create();
  FList := TList.Create();
  FPosition := 0;
end;

destructor TMultiStream.Destroy;
var
  i: Integer;
begin
  for i := FList.Count - 1 downto 0 do
  begin
    GetStream(i).Free();
  end;
  FList.Free();
  inherited Destroy();
end;

function TMultiStream.GetStream(Index: Integer): TStream;
begin
  Result := TStream(FList[Index]);
end;

function TMultiStream.GetTotalSize: Longint;
var
  i: Integer;
begin
  Result := 0;
  for i := 0 to FList.Count - 1 do
  begin
    Result := Result + GetStream(i).Size;
  end;
end;

function TMultiStream.Read(var Buffer; Count: Integer): Longint;
var
  i: Integer;
  buf_pos: PChar;
  len, bytesRead: Longint;
begin
  len := 0;
  Result := 0;
  buf_pos := PChar(@Buffer);
  for i := 0 to FList.Count - 1 do
  begin
    if (FPosition < (len + GetStream(i).Size)) then
    begin
      GetStream(i).Position := FPosition - len;
      bytesRead := GetStream(i).Read(buf_pos^, Count);
      Inc(Result, bytesRead);
      buf_pos := buf_pos + bytesRead;
      Inc(FPosition, bytesRead);
      if (bytesRead < Count) then
      begin
        Dec(Count, bytesRead);
      end else
      begin
        break;
      end;
    end;
    Inc(len, GetStream(i).Size);
  end;
end;

function TMultiStream.Seek(Offset: Integer; Origin: Word): Longint;
var
  len: Integer;
begin
  len := GetTotalSize();
  case Origin of
    soFromBeginning: FPosition := Offset;
    soFromCurrent: FPosition := FPosition + Offset;
    soFromEnd: FPosition := len - Offset;
  end;
  if (FPosition > len) then
  begin
    FPosition := len;
  end else
  if (FPosition < 0) then
  begin
    FPosition := 0;
  end;
  Result := FPosition;
end;

procedure TMultiStream.SetSize(NewSize: Integer);
begin
end;

function TMultiStream.Write(const Buffer; Count: Integer): Longint;
begin
  Result := 0;
end;

end.
